/**
 * This file is in the public domain.
 * You may do whatever you want with it.
 */
package org.calendar;

public class EternalDate {

  public static final String[] DAYS = new String[] {
    "Mo", "Tu", "We", "Th", "Fr", "Sa", "Su"
  };

  public static final String[] DAYS_BG = new String[] {
    "", "", "", "", "", "", ""
  };

  public static final String[] WHOLE_DAYS = new String[] {
    "Monday", "Tuesday", "Wednesday", "Thrusday", "Friday",
    "Saturday", "Sunday"
  };

  public static final String[] WHOLE_DAYS_BG = new String[] {
    "", "", "", "", "",
    "", ""
  };

  public static final String[] MONTHS_NAMES = new String[] {
    "January", "February", "March", "April", "May", "June",
    "July", "August", "September", "October", "November", "December"
  };

  public static final String[] MONTHS_NAMES_BG = new String[] {
    "", "", "", "", "", "",
    "", "", "", "", "", ""
  };

  private static final int[] DAYS_IN_MONTH = new int[] {
    31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
  };

  private static final int DAYS_IN_4_JULIAN_YEARS = 1461;
  private static final int DAYS_IN_400_GREGORIAN_YEARS = 146097;
  private static final int JULIAN_201_OFFSET = 73050;
  private static final int GREGORIAN_101_OFFSET = 36524;
  private static final int GREGORIAN_201_OFFSET = 73048;
  private static final int GREGORIAN_301_OFFSET = 109572;

  private int day = 0;
  private int month = 0;
  private long year = 0;
  private long daysSince201AD = 0;
  private boolean isNewStyle = false;
  private boolean isAD = false;
  private int dayOfWeek = -1;

  public EternalDate(
    int day,
    int month,
    long year,
    boolean isNewStyle,
    boolean isAD
  ) {
    this.day = day;
    this.month = month;
    this.year = year;
    this.isNewStyle = isNewStyle;
    this.isAD = isAD;
    validateDMY();
    calcDays();
    calcDayOfWeek();
  }

  public EternalDate(
    long daysSince201AD,
    boolean isNewStyle
  ) {
    this.daysSince201AD = daysSince201AD;
    this.isNewStyle = isNewStyle;
    calcDMY();
    calcDayOfWeek();
  }

  private void validateDMY() {
    if (year < 1) {
      throw new IllegalArgumentException("The year cannot be less than 1!");
    }
    if (month < 0 || month > 11) {
      throw new IllegalArgumentException("The month must be between 0 and 11!");
    }
    int daysInCurrentMonth = daysInCurrentMonth();
    if (day < 0 || day >= daysInCurrentMonth) {
      throw new IllegalArgumentException("The day must be between 0 and " +
        (daysInCurrentMonth - 1));
    }
  }

  private void calcDays() {
    long numberOfDays = 0;
    if (isAD) {
      if (isNewStyle) {
        numberOfDays = daysADGregorian(year);
      } else {
        numberOfDays = daysADJulian(year);
      }
    } else {
      if (isNewStyle) {
        numberOfDays = daysBCGregorian(year);
      } else {
        numberOfDays = daysBCJulian(year);
      }
    }
    boolean isLeap = isLeap();
    for (int i = 0; i < month; i++) {
      int daysInCurrentMonth = daysInMonth(i, year, isNewStyle, isAD);
      if (isAD) {
        numberOfDays += daysInCurrentMonth;
      } else {
        numberOfDays -= daysInCurrentMonth;
      }
    }
    if (isAD) {
      numberOfDays += day;
    } else {
      numberOfDays -= day;
      numberOfDays = -numberOfDays;
    }
    if (isNewStyle) {
      this.daysSince201AD = numberOfDays - GREGORIAN_201_OFFSET;
    } else {
      this.daysSince201AD = numberOfDays - JULIAN_201_OFFSET;
    }
  }

  private void calcDMY() {
    long numberOfDays = daysSince201AD;
    if (isNewStyle) {
      numberOfDays = daysSince201AD + GREGORIAN_201_OFFSET;
    } else {
      numberOfDays = daysSince201AD + JULIAN_201_OFFSET;
    }
    this.isAD = numberOfDays >= 0;
    if (isAD) {
      calcDateAD(numberOfDays);
    } else {
      if (numberOfDays < -366) {
        // before 1 B.C.
        numberOfDays = - (numberOfDays + 366);
        numberOfDays = calcYearBC(numberOfDays);
      } else {
        // 1 B.C., astronomical year # 0
        this.year = 1;
        numberOfDays = - numberOfDays;
      }
      this.month = 11;
      int daysInMonth = 0;
      for (;;) {
        daysInMonth = daysInMonth(this.month, this.year, this.isNewStyle, this.isAD);
        long nextRem = numberOfDays - daysInMonth;
        if (nextRem < 1) {
          break;
        }
        this.month--;
        numberOfDays = nextRem;
      }
      this.day = daysInMonth - (int)(numberOfDays);
    }
  }

  private long calcYearBC(long numberOfDays) {
    if (isNewStyle) {
      long fhYears = numberOfDays / DAYS_IN_400_GREGORIAN_YEARS;
      long rem     = numberOfDays % DAYS_IN_400_GREGORIAN_YEARS;
      this.year = fhYears * 400;
      if (rem > GREGORIAN_301_OFFSET) {
        this.year += 300;
        rem -= GREGORIAN_301_OFFSET;
      } else if (rem > GREGORIAN_201_OFFSET) {
        this.year += 200;
        rem -= GREGORIAN_201_OFFSET;
      } else if (rem > GREGORIAN_101_OFFSET) {
        this.year += 100;
        rem -= GREGORIAN_101_OFFSET;
      }
      return finishCalculationsBC(rem);
    } else {
      this.year = 0;
      return finishCalculationsBC(numberOfDays);
    }
  }

  private long finishCalculationsBC(long rem) {
    long fYears = rem / DAYS_IN_4_JULIAN_YEARS;
    rem         = rem % DAYS_IN_4_JULIAN_YEARS;
    this.year += (4 * fYears);
    long years = rem / 365;
    rem       = rem % 365;
    this.year += years + 2;
    return rem;
  }

  private void calcDateAD(long numberOfDays) {
    if (isNewStyle) {
      long fhYears = numberOfDays / DAYS_IN_400_GREGORIAN_YEARS;
      long rem     = numberOfDays % DAYS_IN_400_GREGORIAN_YEARS;
      this.year = fhYears * 400;
      if (rem > GREGORIAN_301_OFFSET) {
        this.year += 300;
        rem -= GREGORIAN_301_OFFSET;
      } else if (rem > GREGORIAN_201_OFFSET) {
        this.year += 200;
        rem -= GREGORIAN_201_OFFSET;
      } else if (rem > GREGORIAN_101_OFFSET) {
        this.year += 100;
        rem -= GREGORIAN_101_OFFSET;
      }
      finishCalculations(rem);
    } else {
      this.year = 0;
      finishCalculations(numberOfDays);
    }
  }

  private void finishCalculations(long rem) {
    long fYears = rem / DAYS_IN_4_JULIAN_YEARS;
    rem         = rem % DAYS_IN_4_JULIAN_YEARS;
    this.year += (4 * fYears);
    long years = rem / 365;
    rem       = rem % 365;
    this.year += years + 1;
    this.month = 0;
    for (;;) {
      long nextRem = rem - daysInMonth(this.month, this.year, this.isNewStyle, this.isAD);
      if (nextRem < 0) {
        break;
      }
      this.month++;
      rem = nextRem;
    }
    this.day = (int)rem;
  }

  private void calcDayOfWeek() {
    int firstDay = -1;
    if (isAD) {
      if (isNewStyle) {
        firstDay = firstDayADGregorian(year);
      } else {
        firstDay = firstDayADJulian(year);
      }
    } else {
      if (isNewStyle) {
        firstDay = firstDayBCGregorian(year);
      } else {
        firstDay = firstDayBCJulian(year);
      }
    }
    for (int i = 0; i < month; i++) {
      firstDay = (firstDay + daysInMonth(i, year, isNewStyle, isAD)) % 7;
    }
    firstDay = (firstDay + day) % 7;
    this.dayOfWeek = firstDay;
  }

  public int getDay() {
    return day;
  }

  public long getDaysSince201AD() {
    return daysSince201AD;
  }

  public boolean isAD() {
    return isAD;
  }

  public boolean isNewStyle() {
    return isNewStyle;
  }

  public int getMonth() {
    return month;
  }

  public long getYear() {
    return year;
  }

  public int getDayOfWeek() {
    return dayOfWeek;
  }

  public int compareDaysSince201AD(EternalDate other) {
    if (other == null) {
      return 1;
    }
    if (this == other) {
      return 0;
    }
    if (daysSince201AD < other.daysSince201AD) {
      return -1;
    }
    if (daysSince201AD > other.daysSince201AD) {
      return 1;
    }
    return 0;
  }

  public int compareYears(EternalDate other) {
    if (other == null) {
      return 1;
    }
    if (this == other) {
      return 0;
    }
    if (year < other.year) {
      return -1;
    }
    if (year > other.year) {
      return 1;
    }
    return 0;
  }

  public int compareMonths(EternalDate other) {
    if (this == other) {
      return 0;
    }
    int temp = compareYears(other);
    if (temp != 0) {
      return temp;
    }
    if (month < other.month) {
      return -1;
    }
    if (month > other.month) {
      return 1;
    }
    return 0;
  }

  public int compareDates(EternalDate other) {
    if (this == other) {
      return 0;
    }
    int temp = compareMonths(other);
    if (temp != 0) {
      return temp;
    }
    if (day < other.day) {
      return -1;
    }
    if (day > other.day) {
      return 1;
    }
    return 0;
  }

  public static int firstDayBCJulian(long year) {
    long days = daysBCJulian(year);
    int rem = (int)(days % 7);
    int refFirstDay = 5;
    if (rem <= refFirstDay) {
      refFirstDay -= rem;
    } else {
      refFirstDay += (7 - rem);
    }
    return refFirstDay;
  }

  public static int firstDayADJulian(long year) {
    return (5 + ((int)(daysADJulian(year) % 7))) % 7;
  }

  public static long daysBCJulian(long year) {
    long days = daysADJulian(year);
    days += 366; // 1 B.C. is a leap year.
    return days;
  }

  public static long daysADJulian(long year) {
    year--;
    long rem = year % 4;
    long del = year / 4;
    long goodDays = del * DAYS_IN_4_JULIAN_YEARS;
    return goodDays + (365 * rem);
  }

  public static int firstDayBCGregorian(long year) {
    long days = daysBCGregorian(year);
    int rem = (int)(days % 7);
    int refFirstDay = 0;
    if (rem <= refFirstDay) {
      refFirstDay -= rem;
    } else {
      refFirstDay += (7 - rem);
    }
    return refFirstDay;
  }

  public static int firstDayADGregorian(long year) {
    return (int)(daysADGregorian(year) % 7);
  }

  public static long daysADGregorian(long year) {
    year--;
    long rem = year % 400;
    long del = year / 400;
    return (del * DAYS_IN_400_GREGORIAN_YEARS)
           +
           daysADJulian(rem + 1)
           -
           (rem / 100);
  }

  public static long daysBCGregorian(long year) {
    long days = daysADGregorian(year);
    days += 366; // 1 B.C. is a leap year.
    return days;
  }

  public static boolean isLeap(long year, boolean isNewStyle, boolean isAD) {
    if (!isAD) {
      year--;
    }
    if (isNewStyle) {
      return (((year % 4) == 0) && ((year % 100) != 0)) ||
             ((year % 400) == 0);
    } else {
      return year % 4 == 0;
    }
  }

  public boolean isLeap() {
    return isLeap(year, isNewStyle, isAD);
  }

  public static int daysInMonth(int month, long year, boolean isNewStyle, boolean isAD) {
    int daysInCurrentMonth = DAYS_IN_MONTH[month];
    if (month == 1 && isLeap(year, isNewStyle, isAD)) {
      daysInCurrentMonth++;
    }
    return daysInCurrentMonth;
  }

  public int daysInCurrentMonth() {
    return daysInMonth(month, year, isNewStyle, isAD);
  }

  public String toString() {
    return WHOLE_DAYS[dayOfWeek] + ", " + (day + 1) + " " +
           MONTHS_NAMES[month] +
           ", " + year + " " + (isAD ? "A.D." : "B.C.") +
           (isNewStyle ? " (New Style, Gregorian Calendar)" :
                         " (Old Style, Julian Calendar)");
  }

  public boolean equals(Object x) {
    if (x == null || x.getClass() != getClass()) {
      return false;
    }
    EternalDate date = (EternalDate)x;
    return daysSince201AD == date.daysSince201AD;
  }

  public int hashCode() {
    return (int)((daysSince201AD | 1) * 37);
  }

}