/**
 * This file is in the public domain.
 * You may do whatever you want with it.
 */
package org.calendar;

import java.io.*;
import java.util.*;

public class SimpleDate {

  private volatile transient EternalDate date;
  private volatile transient String text;

  public SimpleDate(
    int day,
    int month,
    long year
  ) {
    this.date = new EternalDate(day, month, year, true, true);
    this.text = null;
  }

  public SimpleDate(
    long daysSince201AD
  ) {
    make(daysSince201AD);
  }

  public SimpleDate() {
    this.date = null;
  }

  private void make(long daysSince201AD) {
    this.date = new EternalDate(daysSince201AD, true);
  }

  public long toLong() {
    return this.date.getDaysSince201AD();
  }

  public int getDay() {
    return this.date.getDay();
  }

  public int getMonth() {
    return this.date.getMonth();
  }

  public long getYear() {
    return this.date.getYear();
  }

  public int daysInCurrentMonth() {
    return this.date.daysInCurrentMonth();
  }

  public int getDayOfWeek() {
    return this.date.getDayOfWeek();
  }

  public SimpleDate getMonthAsDate() {
    if (getDay() == 0) {
      return this;
    }
    return new SimpleDate(0, getMonth(), getYear());
  }

  public SimpleDate getNextMonth() {
    if (this.getYear() >= Long.MAX_VALUE - 9) {
      throw new RuntimeException("The year number is too much!");
    }
    SimpleDate nextMonth = this.getMonth() < 11 ?
      new SimpleDate(0, this.getMonth() + 1, this.getYear()) :
      new SimpleDate(0, 0, this.getYear() + 1);
    return nextMonth;
  }

  public SimpleDate getEndOfMonth() {
    if (this.getYear() >= Long.MAX_VALUE - 9) {
      throw new RuntimeException("The year number is too much!");
    }
    return new SimpleDate(this.daysInCurrentMonth() - 1, this.getMonth(), this.getYear());
  }

  public static SimpleDate todayMonth() {
    Calendar todayUtil = Calendar.getInstance();
    int month = todayUtil.get(Calendar.MONTH);
    long year = todayUtil.get(Calendar.YEAR);
    return new SimpleDate(0, month, year);
  }

  public static SimpleDate todayDate() {
    Calendar todayUtil = Calendar.getInstance();
    int day = todayUtil.get(Calendar.DAY_OF_MONTH) - 1;
    int month = todayUtil.get(Calendar.MONTH);
    long year = todayUtil.get(Calendar.YEAR);
    return new SimpleDate(day, month, year);
  }

  public SimpleDate getDayOfWeekAsDate() {
    return findDayOfWeekInMonth(getDayOfWeek());
  }

  public SimpleDate findDayOfWeekInMonth(int dayOfWeek) {
    SimpleDate month = getMonthAsDate();
    int dayOfFirst = month.getDayOfWeek();
    int day = dayOfWeek - dayOfFirst + 7;
    if (day == getDay()) {
      return this;
    }
    if (day == 0) {
      return month;
    }
    return new SimpleDate(day, getMonth(), getYear());
  }

  public String toString() {
    if (this.text == null) {
      StringBuffer buf = new StringBuffer();
      buf.append(getDay() + 1).append(' ')
         .append(EternalDate.MONTHS_NAMES_BG[getMonth()]).append(' ')
         .append(getYear()).append(',').append(' ')
         .append(EternalDate.WHOLE_DAYS_BG[getDayOfWeek()]);
      this.text = buf.toString();
    }
    return this.text;
  }

  public boolean equals(Object x) {
    if (x == null || x.getClass() != getClass()) {
      return false;
    }
    SimpleDate date = (SimpleDate)x;
    return toLong() == date.toLong();
  }

  public int hashCode() {
    return date.hashCode();
  }

  public static int getDayOfWeekForLong(long daysSince201AD) {
    return (((int)(daysSince201AD % 7)) + 3) % 7;
  }

  public static boolean isWeekEnd(int day) {
    return (day == 5) || (day == 6);
  }

  public static int daysInMonth(int month, long year) {
    return EternalDate.daysInMonth(month, year, true, true);
  }

}