/**
 * This file is in the public domain.
 * You may do whatever you want with it.
 */
package org.finance.data;

import org.calendar.*;
import java.io.*;
import java.util.*;
import org.prevaylerwrapper.*;

public interface Finances extends CommandSystem {

  // mutators:
  public void updateDailyExpense(Payment oldPayment, Payment newPayment);
  public void removeDailyExpense(Payment payment);

  public void updateMonthlyIncome(Payment oldPayment, Payment newPayment);
  public void removeMonthlyIncome(Payment payment);

  public void updatePlannedMonthlyExpense(Payment oldPayment, Payment newPayment);
  public void removePlannedMonthlyExpense(Payment payment);

  public void setPlannedEveryDayExpense(Payment payment);

  public void updatePlannedWeeklyExpense(Payment oldPayment, Payment newPayment);
  public void removePlannedWeeklyExpense(Payment payment);

  public void updatePlannedDailyExpense(long day, DailyPlanning planning);

  // mutators:
  public void updateCardIncome(Payment oldPayment, Payment payment);
  public void removeCardIncome(Payment payment);
  public void updateCardWithdraw(Payment oldPayment, Payment payment);
  public void removeCardWithdraw(Payment payment);

  // balance methods:
  public long getTotalMonthlyIncome(SimpleDate month);
  public long getExpenses(SimpleDate month);
  public long getDailySpentInMonth(SimpleDate month);
  public long getPlannedExpenses(SimpleDate month);
  public long getPlannedBalance(SimpleDate month);
  public long getCurrentBalance(SimpleDate day);
  public long getCurrentPlannedBalance(SimpleDate day);
  public long getDailyBalance(SimpleDate day);
  public long getCardMoney(SimpleDate month);
  public long getCacheMoney(SimpleDate month);
  public long getCurrentMoney(SimpleDate month);

  // display methods:
  public Payment[] getDailyExpenses(SimpleDate month);
  public Payment[] getDailyExpensesSummaries(SimpleDate month);
  public Payment[] getMonthlyIncome(SimpleDate month);
  public Payment[] getPlannedMonthlyExpenses(SimpleDate month);
  public long getPlannedEveryDayExpense(SimpleDate month);
  public long getPlannedWeeklyExpense(SimpleDate day);
  public Payment[] getPlannedWeeklyExpenses(SimpleDate month);
  public Payment[] getPlannedWeeklyExpensesSummaries(SimpleDate month);
  public DailyPlanning[] getPlannedDailyExpenses(SimpleDate month);
  public long getTotalDailyPlanning(SimpleDate day);
  public Payment[] getCardIncome(SimpleDate month);
  public Payment[] getCardIncomeSummaries(SimpleDate month);
  public Payment[] getCardWithdraw(SimpleDate month);
  public Payment[] getCardWithdrawSummaries(SimpleDate month);

}
