/**
 * This file is in the public domain.
 * You may do whatever you want with it.
 */
package org.finance.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.table.*;

import java.util.*;

import org.finance.gui.table.*;
import org.finance.gui.date.*;

public abstract class Finance extends JFrame
implements Updater, ActionListener {

  protected EtchedBorder etchedBorder = new EtchedBorder();

  protected JTabbedPane tabbledPane;
  protected JPanel dailyExpensesPanel;
  protected JPanel incomePanel;
  protected JPanel monthlyExpensesPanel;
  protected JPanel dailyPlanningPanel;
  protected EveryDayExpensePanel everyDayExpensePanel;

  protected JPanel balancePanel;

  protected ExpenseTable expensesTable;
  protected IncomeTable incomeTable;
  protected DailyExpenseTable dailyExpensesTable;
  protected CardIncomeTable cardIncomeTable;
  protected CardWithdrawTable cardWithdrawTable;
  protected DatePanel datePanel;
  protected JPanel mainPanel;
  protected JButton dateSetButton;
  protected JPanel dateHolderPanel;
  protected JLabel currentDateLabel;
  protected JButton todaySetButton;
  protected JPanel dateButtonsPanel;
  protected WeeklyExpenseTable weeklyPlanningTable;
  protected String monthLabelText1 = "   ";
  protected String monthLabelText2 = ", ";
  protected String monthLabelText3 = ". : ";
  protected DailyPlanningTable dailyPlanningTable;

  protected JLabel totalIncomeOutputLabel;
  protected JLabel totalExpensesOutputLabel;
  protected JLabel dailySpentOutputLabel;
  protected JLabel totalPlannedExpensesOutputLabel;
  protected JLabel plannedBalanceOutputLabel;
  protected JLabel currentBalanceOutputLabel;
  protected JLabel currentPlannedBalanceOutputLabel;
  protected JLabel dailyBalanceOutputLabel;
  protected JLabel cardMoneyOutputLabel;
  protected JLabel cacheMoneyOutputLabel;
  protected JLabel allMoneyOutputLabel;

  protected JLabel totalIncomeLabel;
  protected JLabel totalExpensesLabel;
  protected JLabel dailySpentLabel;
  protected JLabel totalPlannedExpensesLabel;
  protected JLabel plannedBalanceLabel;
  protected JLabel currentBalanceLabel;
  protected JLabel currentPlannedBalanceLabel;
  protected JLabel dailyBalanceLabel;
  protected JLabel cardMoneyLabel;
  protected JLabel cacheMoneyLabel;
  protected JLabel allMoneyLabel;

  public Finance() {
    super("");

    everyDayExpensePanel = new EveryDayExpensePanel(this);
    currentDateLabel = new JLabel();
    Font boldFont = new Font(currentDateLabel.getFont().getName(), Font.BOLD, 12);
    currentDateLabel.setFont(boldFont);
    datePanel = new DatePanel(true);
    balancePanel = new JPanel();

    // Labels:
    totalIncomeLabel = new JLabel(" : ", JLabel.RIGHT);
    totalExpensesLabel = new JLabel(" : ", JLabel.RIGHT);
    dailySpentLabel = new JLabel(" : ", JLabel.RIGHT);
    totalPlannedExpensesLabel = new JLabel("  : ", JLabel.RIGHT);
    plannedBalanceLabel = new JLabel(" : ", JLabel.RIGHT);
    currentBalanceLabel = new JLabel(" : ", JLabel.RIGHT);
    currentPlannedBalanceLabel = new JLabel("  : ", JLabel.RIGHT);
    currentPlannedBalanceLabel.setFont(boldFont);
    dailyBalanceLabel = new JLabel("  : ", JLabel.RIGHT);
    cardMoneyLabel = new JLabel("  : ", JLabel.RIGHT);
    cardMoneyLabel.setFont(boldFont);
    cacheMoneyLabel = new JLabel(": ", JLabel.RIGHT);
    cacheMoneyLabel.setFont(boldFont);
    allMoneyLabel = new JLabel(" : ", JLabel.RIGHT);

    // Output Labels:
    totalIncomeOutputLabel = new JLabel("");
    totalExpensesOutputLabel = new JLabel("");
    dailySpentOutputLabel = new JLabel("");
    totalPlannedExpensesOutputLabel = new JLabel("");
    plannedBalanceOutputLabel = new JLabel("");
    currentBalanceOutputLabel = new JLabel("");
    currentPlannedBalanceOutputLabel = new JLabel("");
    currentPlannedBalanceOutputLabel.setFont(boldFont);
    dailyBalanceOutputLabel = new JLabel("");
    cardMoneyOutputLabel = new JLabel("");
    cacheMoneyOutputLabel = new JLabel("");
    allMoneyOutputLabel = new JLabel("");

    cardMoneyOutputLabel.setFont(boldFont);
    cacheMoneyOutputLabel.setFont(boldFont);

    // Tabbed Pane:
    tabbledPane = new JTabbedPane();
    dailyExpensesPanel = new JPanel();
    incomePanel = new JPanel();
    monthlyExpensesPanel = new JPanel();
    dailyExpensesTable = new DailyExpenseTable(this);
    cardIncomeTable = new CardIncomeTable(this);
    cardWithdrawTable = new CardWithdrawTable(this);
    incomeTable = new IncomeTable(this);
    weeklyPlanningTable = new WeeklyExpenseTable(this);
    expensesTable = new ExpenseTable(this);
    dailyPlanningPanel = new JPanel();
    dailyPlanningTable = new DailyPlanningTable(this);

    // Buttons:
    dateSetButton = new JButton(" ");
    todaySetButton = new JButton("");

    // Layout:

    tabbledPane.add(dailyExpensesPanel, " ");
    tabbledPane.add(cardWithdrawTable, "");
    tabbledPane.add(incomePanel, "");
    tabbledPane.add(cardIncomeTable, " ");
    tabbledPane.add(monthlyExpensesPanel, " ");
    tabbledPane.add(weeklyPlanningTable, ".");
    tabbledPane.add(dailyPlanningPanel, ".");
    tabbledPane.add(balancePanel, "");
    tabbledPane.setSelectedIndex(0);
    tabbledPane.setSelectedComponent(dailyExpensesPanel);

    balancePanel.setLayout(new GridBagLayout());

    int y = 0;

    {
      balancePanel.add(totalIncomeLabel, new GridBagConstraints(0, y, 1, 1, 0.0, 0.0,
        GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
      balancePanel.add(totalIncomeOutputLabel, new GridBagConstraints(1, y++, 1, 1, 0.0, 0.0,
        GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    }

    {
      balancePanel.add(totalExpensesLabel, new GridBagConstraints(0, y, 1, 1, 0.0, 0.0,
        GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
      balancePanel.add(totalExpensesOutputLabel, new GridBagConstraints(1, y++, 1, 1, 0.0, 0.0,
        GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    }

    {
      balancePanel.add(dailySpentLabel, new GridBagConstraints(0, y, 1, 1, 0.0, 0.0,
        GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
      balancePanel.add(dailySpentOutputLabel, new GridBagConstraints(1, y++, 1, 1, 0.0, 0.0,
        GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    }

    {
      balancePanel.add(currentBalanceLabel, new GridBagConstraints(0, y, 1, 1, 0.0, 0.0,
        GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
      balancePanel.add(currentBalanceOutputLabel, new GridBagConstraints(1, y++, 1, 1, 0.0, 0.0,
        GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    }

    {
      balancePanel.add(totalPlannedExpensesLabel, new GridBagConstraints(0, y, 1, 1, 0.0, 0.0,
        GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
      balancePanel.add(totalPlannedExpensesOutputLabel, new GridBagConstraints(1, y++, 1, 1, 0.0, 0.0,
        GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    }

    {
      balancePanel.add(plannedBalanceLabel, new GridBagConstraints(0, y, 1, 1, 0.0, 0.0,
        GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
      balancePanel.add(plannedBalanceOutputLabel, new GridBagConstraints(1, y++, 1, 1, 0.0, 0.0,
        GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    }

    {
      balancePanel.add(currentPlannedBalanceLabel, new GridBagConstraints(0, y, 1, 1, 0.0, 0.0,
        GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
      balancePanel.add(currentPlannedBalanceOutputLabel, new GridBagConstraints(1, y++, 1, 1, 0.0, 0.0,
        GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    }

    {
      balancePanel.add(dailyBalanceLabel, new GridBagConstraints(0, y, 1, 1, 0.0, 0.0,
        GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
      balancePanel.add(dailyBalanceOutputLabel, new GridBagConstraints(1, y++, 1, 1, 0.0, 0.0,
        GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    }

    {
      balancePanel.add(cardMoneyLabel, new GridBagConstraints(0, y, 1, 1, 0.0, 0.0,
        GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
      balancePanel.add(cardMoneyOutputLabel, new GridBagConstraints(1, y++, 1, 1, 0.0, 0.0,
        GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    }

    {
      balancePanel.add(cacheMoneyLabel, new GridBagConstraints(0, y, 1, 1, 0.0, 0.0,
        GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
      balancePanel.add(cacheMoneyOutputLabel, new GridBagConstraints(1, y++, 1, 1, 0.0, 0.0,
        GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    }

    {
      balancePanel.add(allMoneyLabel, new GridBagConstraints(0, y, 1, 1, 0.0, 0.0,
        GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
      balancePanel.add(allMoneyOutputLabel, new GridBagConstraints(1, y++, 1, 1, 0.0, 0.0,
        GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    }

    balancePanel.add(new JPanel(), new GridBagConstraints(0, y++, 2, 1, 0.5, 0.5,
      GridBagConstraints.SOUTH, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));

    dailyPlanningPanel.setLayout(new GridBagLayout());
    dailyPlanningPanel.add(dailyPlanningTable, new GridBagConstraints(0, 1, 2, 1, 0.5, 0.5,
      GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH, new Insets(5, 0, 0, 0), 0, 0));
    dailyPlanningPanel.add(everyDayExpensePanel, new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0,
      GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));

    dailyExpensesPanel.setLayout(new GridBagLayout());
    dailyExpensesPanel.add(dailyExpensesTable, new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0,
      GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));

    incomePanel.setLayout(new GridBagLayout());
    incomePanel.add(incomeTable, new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0,
      GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));

    monthlyExpensesPanel.setLayout(new GridLayout(1, 1));
    monthlyExpensesPanel.add(expensesTable);

    mainPanel = new JPanel();
    mainPanel.setLayout(new BorderLayout());

    dateHolderPanel = new JPanel(new BorderLayout());
    dateHolderPanel.add(datePanel, BorderLayout.CENTER);

    dateButtonsPanel = new JPanel();
    dateButtonsPanel.setLayout(new GridBagLayout());
    dateButtonsPanel.add(dateSetButton);
    dateButtonsPanel.add(todaySetButton);

    dateHolderPanel.add(dateButtonsPanel, BorderLayout.EAST);
    dateHolderPanel.add(currentDateLabel, BorderLayout.NORTH);

    ((JPanel)this.getContentPane()).add(mainPanel, BorderLayout.CENTER);

    mainPanel.add(dateHolderPanel, BorderLayout.NORTH);
    mainPanel.add(tabbledPane, BorderLayout.CENTER);

    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        tabbledPane.requestFocus();
      }
    });

    dateSetButton.addActionListener(this);
    todaySetButton.addActionListener(this);
  }

  protected JPanel getTitledPanel(String title) {
    JPanel panel = new JPanel();
    panel.setBorder(new TitledBorder(etchedBorder, title));
    return panel;
  }

}
