/**
 * This file is in the public domain.
 * You may do whatever you want with it.
 */
package org.finance.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

import java.util.*;

import org.calendar.*;
import org.finance.data.*;
import org.prevayler.*;

import java.io.*;

import org.finance.gui.table.*;

public class FinanceLogical extends Finance {

  private static PaymentComparator comparator = new PaymentComparator();
  private static StringComparator scomparator = new StringComparator();

  private Finances finances;

  private SimpleDate today;

  public FinanceLogical() throws Exception {
    this.today = SimpleDate.todayDate();
    this.datePanel.setDate(today);
    finances = FinanceProxy.getFinanceProxy();
    updateBalance();
    fixToday(today);
    fillValues();
  }

  private void updateBalance() {
    totalIncomeOutputLabel.setText(ValueConverter.pureValueToString(finances.getTotalMonthlyIncome(today)));
    totalExpensesOutputLabel.setText(ValueConverter.pureValueToString(finances.getExpenses(today)));
    dailySpentOutputLabel.setText(ValueConverter.pureValueToString(finances.getDailySpentInMonth(today)));
    totalPlannedExpensesOutputLabel.setText(ValueConverter.pureValueToString(finances.getPlannedExpenses(today)));
    plannedBalanceOutputLabel.setText(ValueConverter.pureValueToString(finances.getPlannedBalance(today)));
    currentBalanceOutputLabel.setText(ValueConverter.pureValueToString(finances.getCurrentBalance(today)));
    currentPlannedBalanceOutputLabel.setText(ValueConverter.pureValueToString(finances.getCurrentPlannedBalance(today)));
    dailyBalanceOutputLabel.setText(ValueConverter.pureValueToString(finances.getDailyBalance(today)));
    cardMoneyOutputLabel.setText(ValueConverter.pureValueToString(finances.getCardMoney(today)));
    cacheMoneyOutputLabel.setText(ValueConverter.pureValueToString(finances.getCacheMoney(today)));
    allMoneyOutputLabel.setText(ValueConverter.pureValueToString(finances.getCurrentMoney(today)));
  }

  public void updateDailyPlanningTable() {
    dailyPlanningTable.fillValues();
  }

  private void fillValues() {
    {
      incomeTable.fillValues();
    }
    {
      expensesTable.fillValues();
    }
    {
      dailyExpensesTable.fillValues();
    }
    {
      weeklyPlanningTable.fillValues();
    }
    {
      everyDayExpensePanel.fillValues();
    }
    {
      updateDailyPlanningTable();
    }
    {
      cardIncomeTable.fillValues();
      cardWithdrawTable.fillValues();
    }
  }

  private void fixToday(SimpleDate newDate) {
    SimpleDate oldDate = today;
    if (!oldDate.equals(newDate)) {
      today = newDate;
      updateBalance();
      if (!oldDate.getMonthAsDate().equals(newDate.getMonthAsDate())) {
        fillValues();
      }
    }
    String text = monthLabelText1 +
      EternalDate.MONTHS_NAMES_BG[today.getMonth()] +
      monthLabelText2 +
      today.getYear() +
      monthLabelText3 +
      today.toString();
    this.currentDateLabel.setText(text);
  }

  public void actionPerformed(ActionEvent ae) {
    if (ae.getSource() == dateSetButton) {
      fixToday(datePanel.getDate());
    } else if (ae.getSource() == todaySetButton) {
      datePanel.setDate(SimpleDate.todayDate());
      fixToday(SimpleDate.todayDate());
    }
  }

  public static class PaymentComparator implements Comparator {
    public int compare(Object a, Object b) {
      if (a == b) return 0;
      if (a == null) return 1;
      if (b == null) return -1;
      {
        long x = ((Payment)a).getDateAsLong();
        long y = ((Payment)b).getDateAsLong();
        if (x < y) return -1;
        if (x > y) return 1;
      }
      {
        String x = ((Payment)a).getName();
        String y = ((Payment)b).getName();
        int t = x.compareToIgnoreCase(y);
        if (t != 0) return t;
        return x.compareTo(y);
      }
    }
  }

  public static class StringComparator implements Comparator {
    public int compare(Object a, Object b) {
      if (a == b) return 0;
      if (a == null) return 1;
      if (b == null) return -1;
      String x = (String)a;
      String y = (String)b;
      int t = x.compareToIgnoreCase(y);
      if (t != 0) return t;
      return x.compareTo(y);
    }
  }

  public Comparator getComparator() {
    return comparator;
  }

  public Comparator getSComparator() {
    return scomparator;
  }

  public SimpleDate getToday() {
    return today;
  }

  public void update() {
    updateBalance();
  }

  public Finances getFinances() {
    return finances;
  }

  public void setVisible(boolean vis) {
    super.setVisible(vis);
    if (vis) {
      SwingUtilities.invokeLater(new Runnable() {
        public void run() {
          dailyPlanningTable.fixTableCol();
        }
      });
    }
  }

}
