/**
 * This file is in the public domain.
 * You may do whatever you want with it.
 */
package org.finance.gui.date;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.util.*;

import org.calendar.*;
import org.finance.data.*;
import org.prevayler.*;
import java.io.*;

import org.finance.gui.table.FocusTextField;

public class DatePanel extends JPanel implements ItemListener, DocumentListener {

  private JTextField day = new FocusTextField();
  private JComboBox month = new JComboBox();
  private JTextField year = new FocusTextField();
  private JLabel dayOfWeek = new JLabel();

  private SimpleDate editableMonth = null;

  private Vector listeners = new Vector();
  private SimpleDate lastEvent = null;
  private int editableDay = 1;
  private boolean working = false;

  public DatePanel() {
    this(false);
  }

  public DatePanel(boolean allowOnlyThisMonth) {
    super(new GridLayout(1, 4));
    month.setModel(new DefaultComboBoxModel(EternalDate.MONTHS_NAMES_BG));
    month.setEditable(false);
    month.setSelectedIndex(0);
    day.getDocument().addDocumentListener(this);
    month.addItemListener(this);
    year.getDocument().addDocumentListener(this);
    add(day);
    add(month);
    add(year);
    add(dayOfWeek);
    if (allowOnlyThisMonth) {
      setDate(SimpleDate.todayDate());
      new EditableMonthThread();
    }
  }

  public void setDate(SimpleDate date) {
    working = true;
    try {
      if (lastEvent == null) {
        lastEvent = date;
      }
      day.setText(Integer.toString(date.getDay() + 1));
      month.setSelectedIndex(date.getMonth());
      year.setText(Integer.toString((int)
        Math.min(Math.max(date.getYear(), 1), Integer.MAX_VALUE - 10)));
      makeFixing(date);
    } finally {
      working = false;
    }
  }

  private void makeFixing(SimpleDate date) {
    fixEditableMonth(date, false);
    fixDayOfWeek(date);
  }

  public SimpleDate getDate() {
    int year = getValue(this.year, 1, +1000000000);
    int month = this.month.getSelectedIndex();
    int day = getValue(this.day, 1, SimpleDate.daysInMonth(month, year)) - 1;
    return new SimpleDate(day, month, year);
  }

  private int getValue(JTextField field, int min, int max) {
    String text = field.getText();
    return ValueConverter.stringToInt(text, min, max);
  }

  private void fixDayOfWeek(SimpleDate date) {
    dayOfWeek.setText(EternalDate.WHOLE_DAYS_BG[date.getDayOfWeek()]);
  }

  private void fixEditableMonth(SimpleDate date, boolean allowDayChange) {
    if (editableMonth != null) {
      date = date.getMonthAsDate();
      boolean dayEditingAllowed = date.equals(editableMonth);
      boolean lastWasEditable = lastEvent != null &&
        lastEvent.getMonthAsDate().equals(editableMonth);
      day.setEnabled(dayEditingAllowed);
      if (!dayEditingAllowed) {
        day.setText(Integer.toString(date.daysInCurrentMonth()));
        if (lastWasEditable) {
          editableDay = lastEvent.getDay() + 1;
        }
      } else {
        if (lastWasEditable) {
          editableDay = date.getDay() + 1;
        } else {
          if (allowDayChange) {
            day.setText(Integer.toString(editableDay));
          }
        }
      }
    }
  }

  private void setEditableMonth(SimpleDate editableMonth) {
    working = true;
    try {
      this.editableMonth = editableMonth.getMonthAsDate();
      makeFixing(getDate());
    } finally {
      working = false;
    }
  }

  public void itemStateChanged(ItemEvent e) {
    if (working) return;
    working = true;
    try {
      fireDateChangeAndFixDayOfWeek();
    } finally {
      working = false;
    }
  }

  public void removeUpdate(DocumentEvent e) {
    textChanged();
  }

  public void insertUpdate(DocumentEvent e) {
    textChanged();
  }

  public void changedUpdate(DocumentEvent e) {
    textChanged();
  }

  private void textChanged() {
    if (working) return;
    working = true;
    try {
      fireDateChangeAndFixDayOfWeek();
    } finally {
      working = false;
    }
  }

  public void setEnabled(boolean enabled) {
    working = true;
    try {
      day.setEnabled(enabled);
      month.setEnabled(enabled);
      year.setEnabled(enabled);
    } finally {
      working = false;
    }
  }

  public void setEnabled(
    boolean dayEnabled,
    boolean monthEnabled,
    boolean yearEnabled
  ) {
    working = true;
    try {
      day.setEnabled(dayEnabled);
      month.setEnabled(monthEnabled);
      year.setEnabled(yearEnabled);
    } finally {
      working = false;
    }
  }

  public void addDateChangeListener(DateChangeListener listener) {
    if (this.listeners.indexOf(listener) < 0) {
      this.listeners.addElement(listener);
    }
  }

  private void fireDateChangeAndFixDayOfWeek() {
    SimpleDate date = getDate();
    if (date.equals(lastEvent)) {
      return;
    }
    fixEditableMonth(date, true);
    fixDayOfWeek(date);
    lastEvent = date;
    int size = this.listeners.size();
    for (int i = 0; i < size; i++) {
      DateChangeListener listener = (DateChangeListener)this.listeners.elementAt(i);
      listener.dateChanged(this);
    }
  }

  public void requestFocus() {
    this.day.requestFocus();
  }

  private class EditableMonthThread extends Thread {

    public EditableMonthThread() {
      start();
    }

    public void run() {
      for (;;) {
        SwingUtilities.invokeLater(new Runnable() {
          public void run() {
            setEditableMonth(SimpleDate.todayDate());
          }
        });
        try {
          Thread.sleep(1000);
        } catch (InterruptedException intex) {
          intex.printStackTrace();
        }
      }
    }

  }

}
