/**
 * This file is in the public domain.
 * You may do whatever you want with it.
 */
package org.finance.gui.table;

import org.finance.gui.*;
import org.finance.gui.date.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

import java.util.*;

import org.calendar.*;
import org.finance.data.*;
import org.prevayler.*;
import java.io.*;

import org.finance.gui.table.*;

public class CardIncomeTable extends JPanel
implements ListSelectionListener, ActionListener, DocumentListener, DateChangeListener {

  private Updater updater;

  private JTable cardIncomeTable;
  private JPanel cardIncomeTablePanel;
  private JTable summariesTable;
  private JPanel summariesTablePanel;
  private DatePanel datePanel;
  private JTextField nameField;
  private JTextField moneyField;

  private JButton addButton = new JButton(" ");
  private JButton updateButton = new JButton("");
  private JButton removeButton = new JButton("");

  private JToggleButton summariesButton = new JToggleButton(" ");

  private boolean summaryChanged = true;

  private boolean working = false;
  private int prevRow = -1;

  public CardIncomeTable(Updater updater) {
    super(new BorderLayout());
    JPanel leftPanel = new JPanel(new BorderLayout());
    this.updater = updater;
    this.cardIncomeTable = new JTable();
    this.cardIncomeTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    this.cardIncomeTable.setRowSelectionAllowed(true);
    this.cardIncomeTable.getSelectionModel().addListSelectionListener(this);
    this.cardIncomeTable.setModel(new TableModelImpl(new String[] {
      "", "", ""
    }));
    this.summariesTable = new JTable();
    this.summariesTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    this.summariesTable.setRowSelectionAllowed(true);
    this.summariesTable.getSelectionModel().addListSelectionListener(this);
    this.summariesTable.setModel(new TableModelImpl(new String[] {
      "", ""
    }));
    cardIncomeTablePanel = new JPanel(new BorderLayout());
    cardIncomeTablePanel.add(leftPanel, BorderLayout.CENTER);
    summariesTablePanel = new JPanel(new BorderLayout());
    summariesTablePanel.setVisible(false);
    summariesTablePanel.add(new JScrollPane(summariesTable), BorderLayout.CENTER);
    JPanel tablesHolder = new JPanel(new GridBagLayout());
    tablesHolder.add(cardIncomeTablePanel, new GridBagConstraints(
      0, 0, 1, 1, 1.0, 1.0,
      GridBagConstraints.CENTER,
      GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
    tablesHolder.add(summariesTablePanel, new GridBagConstraints(
      0, 0, 1, 1, 1.0, 1.0,
      GridBagConstraints.CENTER,
      GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
    leftPanel.add(new JScrollPane(cardIncomeTable), BorderLayout.CENTER);
    JPanel rightPanel = new VerticalPanel();
    rightPanel.add(summariesButton);
    rightPanel.add(addButton);
    rightPanel.add(updateButton);
    rightPanel.add(removeButton);
    this.datePanel = new DatePanel();
    this.nameField = new FocusTextField();
    this.moneyField = new FocusTextField();
    this.datePanel.setEnabled(false);
    this.nameField.setEnabled(false);
    this.moneyField.setEnabled(false);
    this.updateButton.setEnabled(false);
    this.removeButton.setEnabled(false);
    this.summariesButton.setEnabled(true);
    JPanel bottomPanel = new JPanel(new GridLayout(2, 1));
    JPanel internalBottomPanel = new JPanel(new GridLayout(1, 2));
    internalBottomPanel.add(this.nameField);
    internalBottomPanel.add(this.moneyField);
    bottomPanel.add(this.datePanel);
    bottomPanel.add(internalBottomPanel);
    leftPanel.add(bottomPanel, BorderLayout.SOUTH);
    this.add(tablesHolder, BorderLayout.CENTER);
    this.add(rightPanel, BorderLayout.EAST);
    addButton.addActionListener(this);
    updateButton.addActionListener(this);
    removeButton.addActionListener(this);
    summariesButton.addActionListener(this);
    nameField.getDocument().addDocumentListener(this);
    moneyField.getDocument().addDocumentListener(this);
    datePanel.addDateChangeListener(this);
  }

  public void fillValues() {
    working = true;
    try {
      Finances finances = updater.getFinances();
      Payment[] cardIncome = finances.getCardIncome(updater.getToday());
      long time = System.currentTimeMillis() ;
      Arrays.sort(cardIncome, updater.getComparator());
      time = System.currentTimeMillis() - time;
      TableModelImpl model = (TableModelImpl)cardIncomeTable.getModel();
      int oldCount = model.getRowCount();
      model.removeAllRows();
      for (int i = 0; i < cardIncome.length; i++) {
        model.addRow(new Object[] {
          cardIncome[i].getDate(),
          cardIncome[i].getName(),
          ValueConverter.pureValueToString(cardIncome[i].getPureValue())
        });
      }
      summaryChanged = true;
      fillSummary();
      makeFixing();
    } catch (Throwable error) {
      error.printStackTrace();
    } finally {
      working = false;
    }
  }

  private void fillSummary() {
    if (!summaryChanged) return;
    Finances finances = updater.getFinances();
    Payment[] cardIncome = finances.getCardIncomeSummaries(updater.getToday());
    long time = System.currentTimeMillis() ;
    Arrays.sort(cardIncome, updater.getComparator());
    time = System.currentTimeMillis() - time;
    TableModelImpl model = (TableModelImpl)summariesTable.getModel();
    int oldCount = model.getRowCount();
    model.removeAllRows();
    for (int i = 0; i < cardIncome.length; i++) {
      model.addRow(new Object[] {
        cardIncome[i].getDate(),
        ValueConverter.pureValueToString(cardIncome[i].getPureValue())
      });
    }
    summaryChanged = false;
  }

  public void valueChanged(ListSelectionEvent e) {
    if (working) return;
    working = true;
    try {
      if (prevRow != getSelectedRow()) {
        makeFixing();
      }
    } catch (Throwable error) {
      error.printStackTrace();
    } finally {
      working = false;
    }
  }

  public void actionPerformed(ActionEvent e) {
    if (working) return;
    working = true;
    try {
      boolean nameFieldRequestFocus = false;
      Object source = e.getSource();
      if (source == addButton) {
        addExpense();
        nameFieldRequestFocus = true;
      } else if (source == updateButton) {
        updateExpense(getSelectedRow());
        nameFieldRequestFocus = true;
      } else if (source == removeButton) {
        removeExpense(getSelectedRow());
      } else if (source == summariesButton) {
        boolean summary = summariesButton.isSelected();
        nameFieldRequestFocus = !summary;
        if (summary) {
          fillSummary();
          cardIncomeTablePanel.setVisible(false);
          summariesTablePanel.setVisible(true);
        } else {
          summariesTablePanel.setVisible(false);
          cardIncomeTablePanel.setVisible(true);
        }
      }
      makeFixing();
      if (nameFieldRequestFocus) {
        nameField.requestFocus();
      }
    } catch (Throwable error) {
      error.printStackTrace();
    } finally {
      working = false;
    }
  }

  private SimpleDate getNewDate() {
    return datePanel.getDate();
  }

  private String getNewName(boolean modify) {
    String text = nameField.getText();
    String result = text.trim();
    if (modify && !result.equals(text)) {
      nameField.setText(result);
    }
    return result;
  }

  private long getNewValue(boolean modify) {
    String text = moneyField.getText();
    long result = ValueConverter.stringToPureValue(text);
    if (modify) {
      String newText = ValueConverter.pureValueToString(result);
      if (!text.equals(newText)) {
        moneyField.setText(newText);
      }
    }
    return result;
  }

  private SimpleDate getOldDate(int row) {
    return (SimpleDate)cardIncomeTable.getModel().getValueAt(row, 0);
  }

  private String getOldName(int row) {
    return (String)cardIncomeTable.getModel().getValueAt(row, 1);
  }

  private long getOldValue(int row) {
    return ValueConverter.stringToPureValue((String)cardIncomeTable.getModel().getValueAt(row, 2));
  }

  private void addExpense() {
    TableModelImpl model = (TableModelImpl)cardIncomeTable.getModel();
    int lastRow = model.getRowCount() - 1;
    boolean doAdd;
    if (lastRow < 0) {
      doAdd = true;
    } else {
      doAdd = getOldName(lastRow).length() > 0;
    }
    if (doAdd) {
      model.addRow(new Object[] {
        updater.getToday(),
        "",
        ValueConverter.pureValueToString(0L)
      });
      lastRow++;
      cardIncomeTable.setRowSelectionInterval(lastRow, lastRow);
    }
  }

  private void swap(int a, int b) {
    for (int i = 0; i < 3; i++) {
      Object t = cardIncomeTable.getModel().getValueAt(a, i);
      cardIncomeTable.getModel().setValueAt(cardIncomeTable.getModel().getValueAt(b, i), a, i);
      cardIncomeTable.getModel().setValueAt(t, b, i);
    }
  }

  private int compare(int a, int b) {
    if (a == b) {
      return 0;
    }
    long dateA = ((SimpleDate)cardIncomeTable.getModel().getValueAt(a, 0)).toLong();
    long dateB = ((SimpleDate)cardIncomeTable.getModel().getValueAt(b, 0)).toLong();
    if (dateA < dateB) {
      return -1;
    }
    if (dateA > dateB) {
      return 1;
    }
    return updater.getSComparator().compare(
      cardIncomeTable.getModel().getValueAt(a, 1),
      cardIncomeTable.getModel().getValueAt(b, 1));
  }

  private void updateExpense(int row) {
    SimpleDate newDate = getNewDate();
    String newName = getNewName(true);
    long newValue = getNewValue(true);
    SimpleDate oldDate = getOldDate(row);
    String oldName = getOldName(row);
    long oldValue = getOldValue(row);
    boolean dateDiff = !newDate.equals(oldDate);
    boolean nameDiff = !newName.equals(oldName);
    if (dateDiff || nameDiff || (newValue != oldValue)) {
      updater.getFinances().updateCardIncome(
        new Payment(oldDate, oldName, oldValue),
        new Payment(newDate, newName, newValue));
      summaryChanged = true;
      updater.update();
      updater.getFinances().print();
    }
    if (dateDiff || nameDiff) {
      TableModelImpl model = (TableModelImpl)cardIncomeTable.getModel();
      cardIncomeTable.getModel().setValueAt(newDate, row, 0);
      cardIncomeTable.getModel().setValueAt(newName, row, 1);
      for (int i = model.getRowCount() - 1; i >= 0; --i) {
        if ((i != row) && (compare(i, row) == 0)) {
          model.removeRow(i);
          if (i < row) {
            row--;
          }
          break;
        }
      }
      while (row > 0 && compare(row - 1, row) > 0) {
        swap(row, row - 1);
        row--;
      }
      while (row < (model.getRowCount() - 1) && compare(row + 1, row) < 0) {
        swap(row, row + 1);
        row++;
      }
      prevRow = row;
    }
    if (newValue != oldValue) {
      cardIncomeTable.getModel().setValueAt(
        ValueConverter.pureValueToString(newValue), row, 2);
    }
    cardIncomeTable.setRowSelectionInterval(row, row);
  }

  private void removeExpense(int row) {
    SimpleDate oldDate = getOldDate(row);
    String oldName = getOldName(row);
    if (oldName.length() > 0) {
      updater.getFinances().removeCardIncome(
        new Payment(oldDate, oldName, getOldValue(row)));
      summaryChanged = true;
      updater.update();
    }
    TableModelImpl model = (TableModelImpl)cardIncomeTable.getModel();
    model.removeRow(row);
  }

  public void removeUpdate(DocumentEvent e) {
    textChanged();
  }

  public void insertUpdate(DocumentEvent e) {
    textChanged();
  }

  public void changedUpdate(DocumentEvent e) {
    textChanged();
  }

  private void textChanged() {
    if (working) return;
    working = true;
    try {
      makeFixing();
    } catch (Throwable error) {
      error.printStackTrace();
    } finally {
      working = false;
    }
  }

  public void dateChanged(Object source) {
    if (working) return;
    working = true;
    try {
      makeFixing();
    } catch (Throwable error) {
      error.printStackTrace();
    } finally {
      working = false;
    }
  }

  private void makeFixing() {
    int row = getSelectedRow();
    boolean addEnabled = true;
    boolean updateEnabled = true;
    boolean removeEnabled = removeButton.isEnabled();
    if (prevRow != row) {
      // if editing:
      if (row >= 0) {
        datePanel.setEnabled(true, false, false);
        nameField.setEnabled(true);
        moneyField.setEnabled(true);
        datePanel.setDate((SimpleDate)cardIncomeTable.getModel().getValueAt(row, 0));
        nameField.setText((String)cardIncomeTable.getModel().getValueAt(row, 1));
        moneyField.setText((String)cardIncomeTable.getModel().getValueAt(row, 2));
        removeEnabled = true;
        nameField.requestFocus();
      } else {
        nameField.setText("");
        moneyField.setText("");
        datePanel.setEnabled(false);
        nameField.setEnabled(false);
        moneyField.setEnabled(false);
        removeEnabled = false;
      }
    }
    if (row >= 0) {
      boolean hasName = nameField.getText().trim().length() > 0;
      if (hasName) {
        SimpleDate newDate = getNewDate();
        String newName = getNewName(false);
        long newValue = getNewValue(false);
        SimpleDate oldDate = getOldDate(row);
        String oldName = getOldName(row);
        long oldValue = getOldValue(row);
        boolean dateDiff = !newDate.equals(oldDate);
        boolean nameDiff = !newName.equals(oldName);
        boolean enabled = dateDiff || nameDiff || (newValue != oldValue);
        updateEnabled = enabled;
      } else {
        updateEnabled = false;
      }
    } else {
      updateEnabled = false;
    }
    boolean summaries = summariesButton.isSelected();
    addButton.setEnabled(addEnabled && !summaries);
    updateButton.setEnabled(updateEnabled && !summaries);
    removeButton.setEnabled(removeEnabled && !summaries);
    prevRow = row;
  }

  private int getSelectedRow() {
    int row = cardIncomeTable.getSelectedRow();
    if (row < 0) return -1;
    if (row >= cardIncomeTable.getModel().getRowCount()) return -1;
    return row;
  }

//  public static void main(String[] arg) throws Exception {
//    final FinanceLogical fl = new FinanceLogical();
//    JFrame f = new JFrame();
//    DailyExpenseTable it = new DailyExpenseTable(fl);
//    it.fillValues();
//    f.getContentPane().add(it);
//    f.setSize(new Dimension(300, 300));
//    f.setLocationRelativeTo(null);
//    f.addWindowListener(new WindowAdapter() {
//      public void windowClosing(WindowEvent e) {
//        fl.getFinances().takeSnapshot();
//        System.exit(0);
//      }
//      public void windowClosed(WindowEvent e) {
//        windowClosing(e);
//      }
//    });
//    f.setVisible(true);
//  }
//
}
