/**
 * This file is in the public domain.
 * You may do whatever you want with it.
 */
package org.finance.gui.table;

import org.finance.gui.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

import java.util.*;

import org.calendar.*;
import org.finance.data.*;
import org.prevayler.*;
import java.io.*;

import org.finance.gui.table.*;

public class DailyPlanningTable extends JPanel
implements ListSelectionListener, ActionListener, DocumentListener {

  private Updater updater;

  private JTable expenseTable;
  private JCheckBox includeDaily;
  private JCheckBox includeWeekly;
  private JTextField moneyField;

  private JButton updateButton = new JButton("");

  private boolean working = false;
  private int prevRow = -1;

  private long dailyValue = -1;
  private Payment[] weeklyExpenses = null;

  private String dailyText = "";
  private String weeklyText = "";

  public DailyPlanningTable(Updater updater) {
    super(new BorderLayout());
    JPanel leftPanel = new JPanel(new BorderLayout());
    this.updater = updater;
    this.expenseTable = new JTable();
    this.expenseTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    this.expenseTable.setRowSelectionAllowed(true);
    this.expenseTable.getSelectionModel().addListSelectionListener(this);
    this.expenseTable.setModel(new TableModelImpl(new String[] {
      "", "", "", "", "  "
    }));
    CheckRenderer checkRenderer = new CheckRenderer();
    this.expenseTable.getColumnModel().getColumn(1).setCellRenderer(checkRenderer);
    this.expenseTable.getColumnModel().getColumn(2).setCellRenderer(checkRenderer);
    leftPanel.add(new JScrollPane(expenseTable), BorderLayout.CENTER);
    JPanel rightPanel = new VerticalPanel();
    rightPanel.add(updateButton);
    this.includeDaily = new JCheckBox(dailyText);
    this.includeWeekly = new JCheckBox(weeklyText);
    this.moneyField = new FocusTextField();
    this.includeDaily.setEnabled(false);
    this.includeWeekly.setEnabled(false);
    this.moneyField.setEnabled(false);
    this.updateButton.setEnabled(false);
    JPanel bottomPanel = new JPanel(new GridLayout(1, 3));
    bottomPanel.add(this.includeDaily);
    bottomPanel.add(this.includeWeekly);
    bottomPanel.add(this.moneyField);
    leftPanel.add(bottomPanel, BorderLayout.SOUTH);
    this.add(leftPanel, BorderLayout.CENTER);
    this.add(rightPanel, BorderLayout.EAST);
    updateButton.addActionListener(this);
    includeDaily.addActionListener(this);
    includeWeekly.addActionListener(this);
    moneyField.getDocument().addDocumentListener(this);
  }

  public void fillValues() {
    working = true;
    try {
      Finances finances = updater.getFinances();
      SimpleDate today = updater.getToday();
      dailyValue = finances.getPlannedEveryDayExpense(today);
      weeklyExpenses = finances.getPlannedWeeklyExpensesSummaries(today);
      Arrays.sort(weeklyExpenses, updater.getComparator());
      DailyPlanning[] dailyPlannings = finances.getPlannedDailyExpenses(today);
      // no sorting here!
      TableModelImpl model = (TableModelImpl)expenseTable.getModel();
      int oldCount = model.getRowCount();
      model.removeAllRows();
      for (int i = 0; i < dailyPlannings.length; i++) {
        SimpleDate day = new SimpleDate(i, today.getMonth(), today.getYear());
        long weeklyExpense = weeklyExpenses[day.getDayOfWeek()].getPureValue();
        model.addRow(new Object[] {
          day,
          getRow(dailyPlannings[i].isDaily(), dailyValue),
          getRow(dailyPlannings[i].isWeekly(), weeklyExpense),
          ValueConverter.pureValueToString(dailyPlannings[i].getPureValue()),
          ValueConverter.pureValueToString(getAll(dailyValue, weeklyExpense, dailyPlannings[i]))
        });
      }
      makeFixing();
    } catch (Throwable error) {
      error.printStackTrace();
    } finally {
      working = false;
    }
  }

  public void fixTableCol() {
    int size = 176 - 10;
    expenseTable.invalidate();
    TableColumn tc = expenseTable.getColumnModel().getColumn(0);
    if (tc != null) {
      tc.setMinWidth(size);
      tc.setPreferredWidth(size);
      tc.setWidth(size);
    }
  }

  private long getAll(long dailyValue, long weeklyExpense, DailyPlanning dailyPlanning) {
    long result = dailyPlanning.getPureValue();
    if (dailyPlanning.isDaily()) {
      result += dailyValue;
    }
    if (dailyPlanning.isWeekly()) {
      result += weeklyExpense;
    }
    return result;
  }

  private Object[] getRow(boolean b, long value) {
    return new Object[] {
      b ? Boolean.TRUE : Boolean.FALSE,
      ValueConverter.pureValueToString(value)};
  }

  private String getValue(int row, int col) {
    Object[] data = (Object[])expenseTable.getModel().getValueAt(row, col);
    return (String)data[1];
  }

  public void valueChanged(ListSelectionEvent e) {
    if (working) return;
    working = true;
    try {
      if (prevRow != getSelectedRow()) {
        makeFixing();
      }
    } catch (Throwable error) {
      error.printStackTrace();
    } finally {
      working = false;
    }
  }

  public void actionPerformed(ActionEvent e) {
    if (working) return;
    working = true;
    try {
      boolean moneyFieldRequestFocus = false;
      Object source = e.getSource();
      if (source == updateButton) {
        updateExpense(getSelectedRow());
        moneyFieldRequestFocus = true;
      }
      makeFixing();
      if (moneyFieldRequestFocus) {
        moneyField.requestFocus();
      }
    } catch (Throwable error) {
      error.printStackTrace();
    } finally {
      working = false;
    }
  }

  private boolean getNewDaily() {
    return this.includeDaily.isSelected();
  }

  private boolean getNewWeekly() {
    return this.includeWeekly.isSelected();
  }

  private long getNewValue(boolean modify) {
    String text = moneyField.getText();
    long result = ValueConverter.stringToPureValue(text);
    if (modify) {
      String newText = ValueConverter.pureValueToString(result);
      if (!text.equals(newText)) {
        moneyField.setText(newText);
      }
    }
    return result;
  }

  private boolean getOldDaily(int row) {
    Object[] data = (Object[])expenseTable.getModel().getValueAt(row, 1);
    return ((Boolean)data[0]).booleanValue();
  }

  private boolean getOldWeekly(int row) {
    Object[] data = (Object[])expenseTable.getModel().getValueAt(row, 2);
    return ((Boolean)data[0]).booleanValue();
  }

  private long getOldValue(int row) {
    return ValueConverter.stringToPureValue((String)expenseTable.getModel().getValueAt(row, 3));
  }

  private static StringBuffer buf = new StringBuffer();

  private static String getCheckBoxText(String base, String value) {
    synchronized (buf) {
      buf.setLength(0);
      buf.append(base).append(' ').append('(').append(value).append(')');
      return buf.toString();
    }
  }

  private void updateExpense(int row) {
    boolean newDaily = getNewDaily();
    boolean newWeekly = getNewWeekly();
    long newValue = getNewValue(true);
    boolean oldDaily = getOldDaily(row);
    boolean oldWeekly = getOldWeekly(row);
    long oldValue = getOldValue(row);
    long oldTotal = ValueConverter.stringToPureValue(
      (String)expenseTable.getModel().getValueAt(row, 4));
    long newTotal = oldTotal;
    SimpleDate day = (SimpleDate)expenseTable.getModel().getValueAt(row, 0);
    if (
      (newDaily != oldDaily) ||
      (newWeekly != oldWeekly) ||
      (newValue != oldValue)
    ) {
      updater.getFinances().updatePlannedDailyExpense(
        day.toLong(), new DailyPlanning(newValue, newDaily, newWeekly));
      updater.update();
      newTotal = updater.getFinances().getTotalDailyPlanning(day);
    }
    if (newDaily != oldDaily) {
      expenseTable.getModel().setValueAt(
        getRow(newDaily, dailyValue),
        row, 1);
    }
    if (newWeekly != oldWeekly) {
      expenseTable.getModel().setValueAt(
        getRow(newWeekly, weeklyExpenses[day.getDayOfWeek()].getPureValue()),
        row, 2);
    }
    if (newValue != oldValue) {
      expenseTable.getModel().setValueAt(
        ValueConverter.pureValueToString(newValue), row, 3);
    }
    if (newTotal != oldTotal) {
      expenseTable.getModel().setValueAt(
        ValueConverter.pureValueToString(newTotal), row, 4);
    }
    expenseTable.setRowSelectionInterval(row, row);
  }

  public void removeUpdate(DocumentEvent e) {
    textChanged();
  }

  public void insertUpdate(DocumentEvent e) {
    textChanged();
  }

  public void changedUpdate(DocumentEvent e) {
    textChanged();
  }

  private void textChanged() {
    if (working) return;
    working = true;
    try {
      makeFixing();
    } catch (Throwable error) {
      error.printStackTrace();
    } finally {
      working = false;
    }
  }

  private void makeFixing() {
    int row = prevRow;
    try {
      row = getSelectedRow();
      if (prevRow != row) {
        // if editing:
        if (row >= 0) {
          moneyField.setText((String)expenseTable.getModel().getValueAt(row, 3));
          includeDaily.setText(getCheckBoxText(dailyText, getValue(row, 1)));
          includeWeekly.setText(getCheckBoxText(weeklyText, getValue(row, 2)));
          includeDaily.setEnabled(true);
          includeWeekly.setEnabled(true);
          includeDaily.setSelected(getOldDaily(row));
          includeWeekly.setSelected(getOldWeekly(row));
          moneyField.setEnabled(true);
          moneyField.requestFocus();
        } else {
          includeDaily.setText(dailyText);
          includeWeekly.setText(weeklyText);
          includeDaily.setSelected(false);
          includeWeekly.setSelected(false);
          moneyField.setText("");
          includeDaily.setEnabled(false);
          includeWeekly.setEnabled(false);
          moneyField.setEnabled(false);
        }
      }
      if (row >= 0) {
        boolean newDaily = getNewDaily();
        boolean newWeekly = getNewWeekly();
        long newValue = getNewValue(false);
        boolean oldDaily = getOldDaily(row);
        boolean oldWeekly = getOldWeekly(row);
        long oldValue = getOldValue(row);
        boolean enabled =
          (newDaily != oldDaily) ||
          (newWeekly != oldWeekly) ||
          (newValue != oldValue);
        updateButton.setEnabled(enabled);
      } else {
        updateButton.setEnabled(false);
      }
    } catch (Throwable error) {
      error.printStackTrace();
    } finally {
      prevRow = row;
    }
  }

  private int getSelectedRow() {
    int row = expenseTable.getSelectedRow();
    if (row < 0) return -1;
    if (row >= expenseTable.getModel().getRowCount()) return -1;
    return row;
  }

  private static JCheckBox checkBox = new JCheckBox();

  static class CheckRenderer extends DefaultTableCellRenderer {
    public Component getTableCellRendererComponent(
      JTable table,
      Object odata,
      boolean isSelected,
      boolean hasFocus,
      int row,
      int col
    ) {
      try {
        Object[] data = (Object[])odata;
        Boolean b = (Boolean)data[0];
        String text = (String)data[1];
        checkBox.setText(text);
        checkBox.setSelected(b.booleanValue());
        DefaultTableCellRenderer component = (DefaultTableCellRenderer)
          super.getTableCellRendererComponent(table, text, isSelected, hasFocus, row, col);
        checkBox.setForeground(component.getForeground());
        checkBox.setBackground(component.getBackground());
        return checkBox;
      } catch (Throwable error) {
        error.printStackTrace(System.out);
        if (error instanceof Error) {
          throw (Error)error;
        }
        if (error instanceof RuntimeException) {
          throw (RuntimeException)error;
        }
        throw new RuntimeException(error);
      }
    }
  }

}
