/**
 * This file is in the public domain.
 * You may do whatever you want with it.
 */
package org.finance.gui.table;

import org.finance.gui.*;
import org.finance.gui.date.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

import java.util.*;

import org.calendar.*;
import org.finance.data.*;
import org.prevayler.*;
import java.io.*;

import org.finance.gui.table.*;

public class EveryDayExpensePanel extends JPanel
implements ActionListener, DocumentListener {

  private Updater updater;

  private JLabel expenseLabel;
  private JLabel dailyExpenseLabel;
  private JTextField dailyExpenseField;

  private String expenseLabelText = ": ";

  private JButton setButton = new JButton(" ");

  private boolean working = false;

  public EveryDayExpensePanel(Updater updater) {
    super(new GridBagLayout());
    this.updater = updater;
    this.expenseLabel = new JLabel();
    JPanel dailyExpensePanel = new JPanel(new GridBagLayout());
    this.dailyExpenseLabel = new JLabel(expenseLabelText);
    this.dailyExpenseField = new FocusTextField();
    dailyExpensePanel.add(this.dailyExpenseLabel, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
      GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    dailyExpensePanel.add(this.dailyExpenseField, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0,
      GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    dailyExpensePanel.add(this.setButton, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0,
      GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    this.add(expenseLabel, new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0,
      GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    this.add(dailyExpensePanel, new GridBagConstraints(0, 1, 1, 1, 1.0, 0.0,
      GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    this.setButton.addActionListener(this);
    this.dailyExpenseField.getDocument().addDocumentListener(this);
  }

  public void fillValues() {
    working = true;
    try {
      long value = getCurrentValue();
      String valueText = ValueConverter.pureValueToString(value);
      expenseLabel.setText(expenseLabelText + valueText);
      dailyExpenseField.setText(valueText);
      makeFixing();
    } catch (Throwable error) {
      error.printStackTrace();
    } finally {
      working = false;
    }
  }

  public void actionPerformed(ActionEvent e) {
    if (working) return;
    working = true;
    try {
      if (e.getSource() == setButton) {
        updater.getFinances().setPlannedEveryDayExpense(
          new Payment(
            updater.getToday(), "", getNewValue()));
        updater.update();
        updater.updateDailyPlanningTable();
        fillValues();
      }
    } finally {
      working = false;
    }
  }

  private long getNewValue() {
    return ValueConverter.stringToPureValue(dailyExpenseField.getText());
  }

  private long getCurrentValue() {
    return updater.getFinances().getPlannedEveryDayExpense(updater.getToday());
  }

  public void removeUpdate(DocumentEvent e) {
    textChanged();
  }

  public void insertUpdate(DocumentEvent e) {
    textChanged();
  }

  public void changedUpdate(DocumentEvent e) {
    textChanged();
  }

  private void textChanged() {
    if (working) return;
    working = true;
    try {
      makeFixing();
    } finally {
      working = false;
    }
  }

  private void makeFixing() {
    long newValue = getNewValue();
    long oldValue = getCurrentValue();
    setButton.setEnabled(newValue != oldValue);
  }

}
