/**
 * This file is in the public domain.
 * You may do whatever you want with it.
 */
package org.finance.gui.table;

import org.finance.gui.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

import java.util.*;

import org.calendar.*;
import org.finance.data.*;
import org.prevayler.*;
import java.io.*;

import org.finance.gui.table.*;

public class IncomeTable extends JPanel
implements ListSelectionListener, ActionListener, DocumentListener {

  private Updater updater;

  private JTable incomeTable;
  private JTextField nameField;
  private JTextField moneyField;

  private JButton addButton = new JButton(" ");
  private JButton updateButton = new JButton("");
  private JButton removeButton = new JButton("");

  private boolean working = false;
  private int prevRow = -1;

  public IncomeTable(Updater updater) {
    super(new BorderLayout());
    JPanel leftPanel = new JPanel(new BorderLayout());
    this.updater = updater;
    this.incomeTable = new JTable();
    this.incomeTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    this.incomeTable.setRowSelectionAllowed(true);
    this.incomeTable.getSelectionModel().addListSelectionListener(this);
    this.incomeTable.setModel(new TableModelImpl(new String[] {
      "", ""
    }));
    leftPanel.add(new JScrollPane(incomeTable), BorderLayout.CENTER);
    JPanel rightPanel = new VerticalPanel();
    rightPanel.add(addButton);
    rightPanel.add(updateButton);
    rightPanel.add(removeButton);
    this.nameField = new FocusTextField();
    this.moneyField = new FocusTextField();
    this.nameField.setEnabled(false);
    this.moneyField.setEnabled(false);
    this.updateButton.setEnabled(false);
    this.removeButton.setEnabled(false);
    JPanel bottomPanel = new JPanel(new GridLayout(1, 2));
    bottomPanel.add(this.nameField);
    bottomPanel.add(this.moneyField);
    leftPanel.add(bottomPanel, BorderLayout.SOUTH);
    this.add(leftPanel, BorderLayout.CENTER);
    this.add(rightPanel, BorderLayout.EAST);
    addButton.addActionListener(this);
    updateButton.addActionListener(this);
    removeButton.addActionListener(this);
    nameField.getDocument().addDocumentListener(this);
    moneyField.getDocument().addDocumentListener(this);
  }

  public void fillValues() {
    working = true;
    try {
      Finances finances = updater.getFinances();
      Payment[] monthlyIncome = finances.getMonthlyIncome(updater.getToday());
      long time = System.currentTimeMillis() ;
      Arrays.sort(monthlyIncome, updater.getComparator());
      time = System.currentTimeMillis() - time;
      TableModelImpl model = (TableModelImpl)incomeTable.getModel();
      int oldCount = model.getRowCount();
      model.removeAllRows();
      for (int i = 0; i < monthlyIncome.length; i++) {
        model.addRow(new Object[] {
          monthlyIncome[i].getName(),
          ValueConverter.pureValueToString(monthlyIncome[i].getPureValue())
        });
      }
      makeFixing();
    } catch (Throwable error) {
      error.printStackTrace();
    } finally {
      working = false;
    }
  }

  public void valueChanged(ListSelectionEvent e) {
    if (working) return;
    working = true;
    try {
      if (prevRow != getSelectedRow()) {
        makeFixing();
      }
    } catch (Throwable error) {
      error.printStackTrace();
    } finally {
      working = false;
    }
  }

  public void actionPerformed(ActionEvent e) {
    if (working) return;
    working = true;
    try {
      boolean nameFieldRequestFocus = false;
      Object source = e.getSource();
      if (source == addButton) {
        addIncome();
        nameFieldRequestFocus = true;
      } else if (source == updateButton) {
        updateIncome(getSelectedRow());
        nameFieldRequestFocus = true;
      } else if (source == removeButton) {
        removeIncome(getSelectedRow());
      }
      makeFixing();
      if (nameFieldRequestFocus) {
        nameField.requestFocus();
      }
    } catch (Throwable error) {
      error.printStackTrace();
    } finally {
      working = false;
    }
  }

  private String getNewName(boolean modify) {
    String text = nameField.getText();
    String result = text.trim();
    if (modify && !result.equals(text)) {
      nameField.setText(result);
    }
    return result;
  }

  private long getNewValue(boolean modify) {
    String text = moneyField.getText();
    long result = ValueConverter.stringToPureValue(text);
    if (modify) {
      String newText = ValueConverter.pureValueToString(result);
      if (!text.equals(newText)) {
        moneyField.setText(newText);
      }
    }
    return result;
  }

  private String getOldName(int row) {
    return (String)incomeTable.getModel().getValueAt(row, 0);
  }

  private long getOldValue(int row) {
    return ValueConverter.stringToPureValue((String)incomeTable.getModel().getValueAt(row, 1));
  }

  private void addIncome() {
    TableModelImpl model = (TableModelImpl)incomeTable.getModel();
    int lastRow = model.getRowCount() - 1;
    boolean doAdd;
    if (lastRow < 0) {
      doAdd = true;
    } else {
      doAdd = getOldName(lastRow).length() > 0;
    }
    if (doAdd) {
      model.addRow(new Object[] {
        "",
        ValueConverter.pureValueToString(0L)
      });
      lastRow++;
      incomeTable.setRowSelectionInterval(lastRow, lastRow);
    }
  }

  private void swap(int a, int b) {
    for (int i = 0; i < 2; i++) {
      Object t = incomeTable.getModel().getValueAt(a, i);
      incomeTable.getModel().setValueAt(incomeTable.getModel().getValueAt(b, i), a, i);
      incomeTable.getModel().setValueAt(t, b, i);
    }
  }

  private void updateIncome(int row) {
    String newName = getNewName(true);
    long newValue = getNewValue(true);
    String oldName = getOldName(row);
    long oldValue = getOldValue(row);
    boolean nameDiff = !newName.equals(oldName);
    if (nameDiff || (newValue != oldValue)) {
      updater.getFinances().updateMonthlyIncome(
        new Payment(updater.getToday(), oldName, oldValue),
        new Payment(updater.getToday(), newName, newValue));
      updater.update();
    }
    if (nameDiff) {
      TableModelImpl model = (TableModelImpl)incomeTable.getModel();
      for (int i = model.getRowCount() - 1; i >= 0; --i) {
        if (i != row && newName.equals(incomeTable.getModel().getValueAt(i, 0))) {
          model.removeRow(i);
          if (i < row) {
            row--;
          }
          break;
        }
      }
      Comparator comparator = updater.getSComparator();
      while (row > 0 && comparator.compare(
        incomeTable.getModel().getValueAt(row - 1, 0), newName) > 0
      ) {
        swap(row, row - 1);
        row--;
      }
      while (row < (model.getRowCount() - 1) && comparator.compare(
        incomeTable.getModel().getValueAt(row + 1, 0), newName) < 0
      ) {
        swap(row, row + 1);
        row++;
      }
      prevRow = row;
      incomeTable.getModel().setValueAt(newName, row, 0);
    }
    if (newValue != oldValue) {
      incomeTable.getModel().setValueAt(
        ValueConverter.pureValueToString(newValue), row, 1);
    }
    incomeTable.setRowSelectionInterval(row, row);
  }

  private void removeIncome(int row) {
    String oldName = getOldName(row);
    updater.getFinances().removeMonthlyIncome(
      new Payment(updater.getToday(), oldName, getOldValue(row)));
    updater.update();
    TableModelImpl model = (TableModelImpl)incomeTable.getModel();
    model.removeRow(row);
  }

  public void removeUpdate(DocumentEvent e) {
    textChanged();
  }

  public void insertUpdate(DocumentEvent e) {
    textChanged();
  }

  public void changedUpdate(DocumentEvent e) {
    textChanged();
  }

  private void textChanged() {
    if (working) return;
    working = true;
    try {
      makeFixing();
    } catch (Throwable error) {
      error.printStackTrace();
    } finally {
      working = false;
    }
  }

  private void makeFixing() {
    int row = getSelectedRow();
    if (prevRow != row) {
      // if editing:
      if (row >= 0) {
        nameField.setEnabled(true);
        moneyField.setEnabled(true);
        nameField.setText((String)incomeTable.getModel().getValueAt(row, 0));
        moneyField.setText((String)incomeTable.getModel().getValueAt(row, 1));
        removeButton.setEnabled(true);
        nameField.requestFocus();
      } else {
        nameField.setText("");
        moneyField.setText("");
        nameField.setEnabled(false);
        moneyField.setEnabled(false);
        removeButton.setEnabled(false);
      }
    }
    if (row >= 0) {
      boolean hasName = nameField.getText().trim().length() > 0;
      if (hasName) {
        String newName = getNewName(false);
        long newValue = getNewValue(false);
        String oldName = getOldName(row);
        long oldValue = getOldValue(row);
        boolean nameDiff = !newName.equals(oldName);
        boolean enabled = nameDiff || (newValue != oldValue);
        updateButton.setEnabled(enabled);
      } else {
        updateButton.setEnabled(false);
      }
    } else {
      updateButton.setEnabled(false);
    }
    prevRow = row;
  }

  private int getSelectedRow() {
    int row = incomeTable.getSelectedRow();
    if (row < 0) return -1;
    if (row >= incomeTable.getModel().getRowCount()) return -1;
    return row;
  }

//  public static void main(String[] arg) throws Exception {
//    final FinanceLogical fl = new FinanceLogical();
//    JFrame f = new JFrame();
//    IncomeTable it = new IncomeTable(fl);
//    it.fillValues();
//    f.getContentPane().add(it);
//    f.setSize(new Dimension(300, 300));
//    f.setLocationRelativeTo(null);
//    f.addWindowListener(new WindowAdapter() {
//      public void windowClosing(WindowEvent e) {
//        fl.getFinances().takeSnapshot();
//        System.exit(0);
//      }
//      public void windowClosed(WindowEvent e) {
//        windowClosing(e);
//      }
//    });
//    f.setVisible(true);
//  }
//
}
