/**
 * This file is in the public domain.
 * You may do whatever you want with it.
 */
package org.finance.gui.table;

import javax.swing.table.*;
import javax.swing.event.*;
import java.util.*;

public class TableModelImpl extends AbstractTableModel {

  private String[] columnNames;
  private Vector rowData = new Vector();

  public TableModelImpl(String[] columnNames) {
    int size = columnNames != null ? columnNames.length : 0;
    this.columnNames = new String[size];
    for (int i = 0; i < size; ++i) {
      this.columnNames[i] = columnNames[i];
    }
  }

  public String getColumnName(int col) {
    return columnNames[col];
  }

  public int getRowCount() {
    return rowData.size();
  }

  public int getColumnCount() {
    return columnNames.length;
  }

  public Object getValueAt(int row, int col) {
    return ((Object[])rowData.elementAt(row))[col];
  }

  public boolean isCellEditable(int row, int col) {
    return false;
  }

  public void setValueAt(Object value, int row, int col) {
    ((Object[])rowData.elementAt(row))[col] = value;
    fireTableCellUpdated(row, col);
  }

  public void addRow(Vector data) {
    int size = data != null ? data.size() : 0;
    Object[] row = new Object[size];
    for (int i = 0; i < size; ++i) {
      row[i] = data.elementAt(i);
    }
    int rowNum = rowData.size();
    rowData.addElement(row);
    fireTableRowsInserted(rowNum, rowNum);
  }

  public void addRow(Object[] data) {
    int size = data != null ? data.length : 0;
    Object[] row = new Object[size];
    for (int i = 0; i < size; ++i) {
      row[i] = data[i];
    }
    int rowNum = rowData.size();
    rowData.addElement(row);
    fireTableRowsInserted(rowNum, rowNum);
  }

  public void removeAllRows() {
    rowData.removeAllElements();
    fireTableDataChanged();
  }

  public void removeRow(int i) {
    rowData.removeElementAt(i);
    fireTableRowsDeleted(i, i);
  }


}