/**
 * This file is in the public domain.
 * You may do whatever you want with it.
 */
package org.prevaylerwrapper;

import org.prevayler.*;
import java.lang.reflect.*;
import org.prevayler.implementation.*;
import java.io.*;

public final class CommandProxy implements InvocationHandler {

  private SnapshotPrevayler prevayler;

  private CommandProxy(
    String commandSystemClassName,
    String dataDirectory,
    boolean deleteOld
  ) throws Exception {
    Data data = new Data(commandSystemClassName);
    this.prevayler = new SnapshotPrevayler(
      data, "PrevalenceBase/" + dataDirectory, deleteOld);
  }

  public static CommandSystem newProxyInstance(
    Class iface,
    String commandSystemClassName,
    String dataDirectory,
    boolean deleteOld
  ) throws Exception {
    return (CommandSystem)Proxy.newProxyInstance(
      iface.getClassLoader(),
      new Class[] {iface},
      new CommandProxy(commandSystemClassName, dataDirectory, deleteOld)
    );
  }

  public Object invoke(Object x, Method method, Object[] args) {
    try {
      Data data = (Data)prevayler.system();
      String name = method.getName();
      if ("print".equals(name)) {
        data.print();
      } else if ("takeSnapshot".equals(name)) {
        System.out.println("taking snapshot");
        prevayler.takeSnapshot();
      } else if (
        name.startsWith("add") ||
        name.startsWith("set") ||
        name.startsWith("remove") ||
        name.startsWith("update")
      ) {
        System.out.println("executing command : " + name);
        prevayler.executeCommand(new CommandProxyCommand(name, args, method.getParameterTypes()));
      } else if (
        name.startsWith("get")
      ) {
        System.out.println("invoking directly : " + name);
        return method.invoke(data.getSystem(), args);
      } else {
        throw new RuntimeException("Cannot call method : " + name +
          "\nthe name must start with add, set, remove, or update!");
      }
    } catch (Exception ex) {
      ex.printStackTrace();
    }
    return null;
  }

}