/**
 * This file is in the public domain.
 * You may do whatever you want with it.
 */
package org.prevaylerwrapper;

import java.util.*;
import org.prevayler.*;
import org.prevayler.implementation.AbstractPrevalentSystem;
import java.io.*;
import java.lang.reflect.*;

public class Data extends AbstractPrevalentSystem {

  private Hashtable data = new Hashtable();
  private String commandSystemClassName = null;
  private transient CommandSystem system = null;

  public Data(String commandSystemClassName) {
    this.commandSystemClassName = commandSystemClassName;
  }

  public void put(String key, Serializable value) {
    data.put(key, value);
  }

  public Serializable get(String key) {
    return (Serializable)data.get(key);
  }

  public void purge(String[] keys) {
    Hashtable newData = new Hashtable();
    for (Enumeration keysEnum = data.keys(); keysEnum.hasMoreElements(); ) {
      String key = (String)keysEnum.nextElement();
      if (isIn(key, keys)) {
        newData.put(key, data.get(key));
      }
    }
    data = newData;
  }

  private static boolean isIn(String key, String[] keys) {
    for (int i = 0; i < keys.length; i++) {
      String k = keys[i];
      if (key == k || key.equals(k)) {
        return true;
      }
    }
    return false;
  }

  public CommandSystem getSystem() {
    if (this.system == null) {
      try {
        Class c = Class.forName(commandSystemClassName);
        Class[] none = new Class[0];
        Constructor constructor = c.getConstructor(none);
        this.system = (CommandSystem)constructor.newInstance(none);
        this.system.setData(this);
      } catch (Throwable error) {
        error.printStackTrace();
      }
    }
    return this.system;
  }

  public void print() {
    System.out.println("data = " + data);
    System.out.println("system = " + system);
  }

}